/*--------------------------------------------------------------------------
 Copyright 1999,2001, Dan Kegel http://www.kegel.com/
 Copyright 2003 Ixia http://www.ixiacom.com/
 See the file COPYING

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--------------------------------------------------------------------------*/

#ifndef LIBRN_H
#define LIBRN_H

#include <poll.h>

#define rn_POLLIN  POLLIN
#define rn_POLLOUT POLLOUT
#define rn_POLLERR POLLERR
#define rn_POLLHUP POLLHUP


typedef struct {
	char	_data[144];
} rn_t;


struct rn_pollevent_s;

typedef int (*rn_callback_fn_t)	(struct rn_pollevent_s	*event);


struct rn_client_s {
	/* The callback fn */
	rn_callback_fn_t pfn;

	/* The data to be passed back */
	void		*data;
}; 
typedef struct rn_client_s	rn_client_t;


struct rn_pollevent_s {
	/* the file descriptor that is ready for I/O (same as in struct pollfd). */
	int		fd;

	/* the kind of I/O this fd is currently ready for (same as in struct pollfd). */
	short		revents;

	/* the object to call to handle I/O on this fd. */
	rn_client_t	client;
};

typedef struct rn_pollevent_s	rn_pollevent_t;


#ifdef __cplusplus
extern "C" {
#endif

/* The function pointers apps use to access the rn_t methods.
 * I used to use #define's to do this indirection without runtime penalty,
 * but it turns out to be impossible to tell at compile time which of
 * the schemes is best to use, so I now pick one at runtime.
 */

/** Tell the rn_t which signal number to use. Call once after init(), before add(). */
extern int 	(*rn_setSignum)			(rn_t 			*,
						 int			signum);


/** Initialize the rn_t. */
extern int 	(*rn_init)			(rn_t			*,
						 int			maxfds);


/** Release any resouces allocated internally by this rn_t. */
extern void	(*rn_shutdown)			(rn_t			*);


/**
 Sets the flags of fd for use with sigio
 On exit, the fd is ready for use with rn_add
 Note: Overrides the old flags of fd.
 @param fd - the file descriptor to be used for IO
 @param pid - the pid of the process. 
 @return 0 on success, errno on failure
*/
extern int	(*rn_prepare_fd_for_add)	(int			fd,
						 int			pid);


/**
 Add a file descriptor to the set we monitor. 
 Caller should already have established a handler for SIGIO.
 @param fd file descriptor to add
 @param pfn function to call when fd is ready
 @param data context pointer to pass to pfn
 */
extern int	(*rn_add)			(rn_t			*,
						 int			fd,
						 rn_callback_fn_t	pfn,
						 void			*data);


/** Remove a file descriptor. */
extern int	(*rn_del)			(rn_t			*,
						 int			fd);


/**
 Sleep at most timeout_millisec waiting for an I/O readiness event
 on the file descriptors we're watching, and dispatch events to
 the handler for each file descriptor that is ready for I/O.
 This is included as an example of how to use
 waitForEvents and getNextEvent.  Real programs should probably
 avoid waitAndDispatchEvents and call waitForEvents and getNextEvent
 instead for maximum control over client deletion.
 */
extern int (*rn_waitAndDispatchEvents)(rn_t *, int timeout_millisec);

#ifdef __cplusplus
}
#endif

#endif	/* !LIBRN_H */
